# -*- coding: utf-8 -*-
#
# UnivNautes
# Copyright (C) 2014  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import xml.etree.ElementTree as ET
from base64 import b64decode

from django.conf import settings

def root():
    with open(settings.CONFIG_XML,'r') as f:
        x = ET.fromstring(f.read())
    return x

def laxint(s):
    try:
        return int(s)
    except:
        return 0

def get_ca(caref):
    for ca in root().findall('ca'):
        if ca.find('refid').text == caref:
            crt = ca.find('crt')
            if crt is not None:
                crt = crt.text.decode('base64')
            prv = ca.find('prv')
            if prv is not None:
                prv = prv.text.decode('base64')
            return {
                    'caref': caref,
                    'crt': crt,
                    'prv': prv,
                    }
    return None

def get_cert(certref):
    for cert in root().findall('cert'):
        if cert.find('refid').text == certref:
            crt = cert.find('crt')
            if crt is not None:
                crt = crt.text.decode('base64')
            prv = cert.find('prv')
            if prv is not None:
                prv = prv.text.decode('base64')
            caref = cert.find('caref')
            if caref is not None:
                caref = caref.text
            descr = cert.find('descr')
            if descr is not None:
                descr = descr.text
            return {
                    'certref': caref,
                    'descr': descr,
                    'caref': caref,
                    'crt': crt,
                    'prv': prv,
                    }
    return None

def get_idp():
    idp = root().find('univnautes/idp')
    if idp is None:
        return None

    if idp.find('enable') is None:
        return None

    certref = idp.find('certref')
    if certref is not None:
        saml_cert = get_cert(certref.text)
    else:
        saml_cert = None

    return { 'saml_cert': saml_cert }

def get_federations():
    """
    <federation>
        <enable/>
        <codename>renater_test</codename>
        <refid>fed_53d1161955a26</refid>
        <descr><![CDATA[Renater TEST Federation]]></descr>
        <url>https://federation.renater.fr/test/renater-test-metadata.xml</url>
        <metadata>[base64 encoded metadata]</metadata>
        <certref>53d115fac567b</certref>
    </federation>
    """
    xml_federations = root().find('univnautes/federations')
    if xml_federations is None:
        return []
    xml_federations = xml_federations.findall('federation')
    federations = []
    for xml_federation in xml_federations:
        if xml_federation.find('enable') is None:
            continue
        codename = xml_federation.find('codename')
        if codename is not None:
            codename = codename.text
        url = xml_federation.find('url')
        if url is not None:
            url = url.text
        metadata = xml_federation.find('metadata')
        if metadata is not None:
            try:
                metadata = b64decode(metadata.text)
            except:
                metadata = None
        descr = xml_federation.find('descr')
        if descr is not None:
            descr = descr.text
        certref = xml_federation.find('certref')
        if certref is not None:
            signcert = (get_cert(certref.text) or {}).get('crt')
        else:
            signcert = None
        federations.append({
            'codename': codename,
            'url': url,
            'metadata': metadata,
            'signcert': signcert,
            'descr': descr,
            })
    return federations
