#!/usr/local/bin/bash

DB=/var/db/univnautes-idp.sqlite3
IDPDIR=/usr/local/univnautes/idp

cd $IDPDIR

function cronstop() {
    for cron in idp-update-metadatas
    do
        if [ -r /var/run/${cron}-cron.pid ]
        then
            PID=`cat /var/run/${cron}-cron.pid`
            ps waux | grep "$PID" | grep minicron | grep -vq grep && kill $PID
            rm -f /var/run/${cron}-cron.pid
        fi
    done
}

function cronstart() {
    cronstop
    /usr/local/bin/minicron 3600 /var/run/idp-update-metadatas-cron.pid $IDPDIR/idp-update-metadatas.sh
}

function lightystop() {
    if [ -r /var/run/lighty-idp.pid ]
    then
        PID=`cat /var/run/lighty-idp.pid`
        ps waux | grep "$PID" | grep lighty-idp | grep -vq grep && kill $PID
        rm -f /var/run/lighty-idp.pid
    fi
}

function lightystart() {
    lightystop
    /usr/local/sbin/lighttpd -f /usr/local/univnautes/idp/lighty-idp.conf
}


function syncdata() {
    echo "sync metadatas in progress (backgrounded)" | logger -p local4.info -t idp/syncdata
    (
      cd /usr/local/univnautes/idp/
      ./idp-update-metadatas.sh | logger -p local4.info -t idp/idp-update-metadatas
    ) &
}

function syncdb() {
    if ! test -r $DB
    then
        python manage.py syncdb --noinput --no-initial-data | logger -p local4.info -t idp/syncdb
        python manage.py loaddata fixtures/* | logger -p local4.info -t idp/loaddata
    fi
}

function start() {
    if python manage.py configxml get idp > /dev/null
    then
        syncdb
        python manage.py collectstatic -v0 -l --noinput | logger -p local4.info -t idp/collectstatic
        python manage.py runfcgi socket=/tmp/univnautes-idp-fcgi.sock method=prefork daemonize=true pidfile=/var/run/univnautes-idp-fcgi.pid
        echo "started (manage.py runfcgi)" | logger -p local4.info -t idp/start
        syncdata
        cronstart
        lightystart
    fi
}

function stop() {
    lightystop
    cronstop
    kill $(cat /var/run/univnautes-idp-fcgi.pid)
    echo "stopped (kill)" | logger -p local4.info -t idp/stop
}

function restart() {
    stop
    sleep 1
    start
}

function status() {
    ps waux | grep $(cat /var/run/univnautes-idp-fcgi.pid) |  grep -v grep
}

echo $1 | logger -p local4.info -t idp/rc

$1

